using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.Text;
using Vevo.DataAccessLib;
using Vevo.Domain;
using Vevo.Domain.Base;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.Contents;
using Vevo.Shared.DataAccess;
using Vevo.Shared.Utilities;
using Vevo.Base.Data;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class ContentRepositoryNoCache : IContentRepository
    {
        #region Private

        private const string AllFields =
            " Content.ContentID, ContentName, Title, IsEnabled, IsShowInSiteMap, UrlName,  Other1, Other2, Other3, CultureID, Body, MetaTitle, MetaKeyword, MetaDescription, SubscriptionLevelID ";

        private const string AllContentLocaleFields = " ContentID, CultureID, Title, Body, MetaTitle, MetaKeyword, MetaDescription, UrlName ";

        private UrlNameAccessHelper _urlNameHelper = new UrlNameAccessHelper( "ContentLocale", "ContentID" );

        private string CreateSortByText( string sortBy )
        {
            if (sortBy == null || sortBy.Trim() == "")
                return "";
            else if (sortBy.Trim().ToLower().StartsWith( "contentid" ))
                return "ORDER BY Content." + sortBy.Trim() + " ";
            else if (sortBy.Trim().ToLower().StartsWith( "title" ))
                return "ORDER BY L." + sortBy.Trim() + " ";
            else
                return "ORDER BY " + sortBy + " ";
        }

        private string ConvertFieldName( string fieldName )
        {
            switch (fieldName.ToLower())
            {
                case "contentid":
                    return "Content.ContentID";
                default:
                    return fieldName;
            }
        }

        private ContentLocale CreateOneLocaleFromRow( DataRow row )
        {
            ContentLocale locale = new ContentLocale();
            locale.CultureID = row["CultureID"].ToString();
            locale.Title = row["Title"].ToString();
            locale.Body = row["Body"].ToString();
            locale.MetaTitle = row["MetaTitle"].ToString();
            locale.MetaKeyword = row["MetaKeyword"].ToString();
            locale.MetaDescription = row["MetaDescription"].ToString();
            locale.UrlName = row["UrlName"].ToString();

            return locale;
        }

        private List<ILocale> ConvertLocaleTableToList( DataTable localeTable )
        {
            List<ILocale> list = new List<ILocale>();

            foreach (DataRow row in localeTable.Rows)
                list.Add( CreateOneLocaleFromRow( row ) );

            return list;
        }

        private void DeleteLocales( string contentID )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE FROM ContentLocale " +
                "WHERE ContentID = @ContentID; ",
                DataAccess.CreateParameterString( contentID ) );
        }

        private void InsertLocales( Content content )
        {
            foreach (ContentLocale locale in content.Locales)
            {
                locale.UrlName = _urlNameHelper.CreateLocaleUrlName( locale.UrlName );
                DataAccess.ExecuteNonQuery(
                "INSERT INTO ContentLocale ( ContentID, CultureID, Title, Body, MetaTitle, MetaKeyword, MetaDescription, UrlName ) " +
                "VALUES ( @ContentID, @CultureID, @Title, @Body, @MetaTitle, @MetaKeyword, @MetaDescription, @UrlName ); " +
                "SELECT @@Identity; ",
                DataAccess.CreateParameterString( content.ContentID ),
                DataAccess.CreateParameterString( locale.CultureID ),
                DataAccess.CreateParameterString( locale.Title ),
                DataAccess.CreateParameterString( locale.Body ),
                DataAccess.CreateParameterString( locale.MetaTitle ),
                DataAccess.CreateParameterString( locale.MetaKeyword ),
                DataAccess.CreateParameterString( locale.MetaDescription ),
                DataAccess.CreateParameterString( locale.UrlName )
                );
            }
        }

        private Content Create( Content content )
        {
            string contentID = DataAccess.ExecuteScalar(
               "INSERT INTO Content ( ContentName, IsShowInSiteMap, IsEnabled, Other1, Other2, Other3, SubscriptionLevelID ) " +
               "VALUES ( @contentName, @IsShowInSiteMap, @IsEnabled, @Other1, @Other2, @Other3, @SubscriptionLevelID ); " +
               "SELECT @@Identity; ",
               DataAccess.CreateParameterString( content.ContentName ),
               DataAccess.CreateParameterBool( content.IsShowInSiteMap ),
               DataAccess.CreateParameterBool( content.IsEnabled ),
               DataAccess.CreateParameterString( content.Other1 ),
               DataAccess.CreateParameterString( content.Other2 ),
               DataAccess.CreateParameterString( content.Other3 ),
               DataAccess.CreateParameterString( content.SubscriptionLevelID )
               );

            content.ContentID = contentID;

            InsertLocales( content );


            return content;
        }

        private Content Update( Content content )
        {
            DataAccess.ExecuteNonQuery( "UPDATE Content " +
                "SET ContentName = @ContentName, " +
                "IsShowInSiteMap=@IsShowInSiteMap, IsEnabled=@IsEnabled, " +
                "Other1=@Other1, Other2=@Other2, Other3=@Other3, SubscriptionLevelID = @SubscriptionLevelID " +
                "WHERE ContentID = @ContentID; ",
               DataAccess.CreateParameterString( content.ContentName ),
               DataAccess.CreateParameterBool( content.IsShowInSiteMap ),
               DataAccess.CreateParameterBool( content.IsEnabled ),
               DataAccess.CreateParameterString( content.Other1 ),
               DataAccess.CreateParameterString( content.Other2 ),
               DataAccess.CreateParameterString( content.Other3 ),
               DataAccess.CreateParameterString( content.SubscriptionLevelID ),
               DataAccess.CreateParameterString( content.ContentID ) );

            DeleteLocales( content.ContentID );
            InsertLocales( content );


            return content;
        }

        private Content CreateOneFromRow( Culture culture, DataRow row )
        {
            Content content = new Content( culture );

            content.ContentID = row["ContentID"].ToString();
            content.ContentName = row["ContentName"].ToString();
            content.Title = row["Title"].ToString();
            content.Body = row["Body"].ToString();
            content.IsEnabled = ConvertUtilities.ToBoolean( row["IsEnabled"] );
            content.IsShowInSiteMap = ConvertUtilities.ToBoolean( row["IsShowInSiteMap"] );
            content.Other1 = row["Other1"].ToString();
            content.Other2 = row["Other2"].ToString();
            content.Other3 = row["Other3"].ToString();
            content.MetaTitle = row["MetaTitle"].ToString();
            content.MetaKeyword = row["MetaKeyword"].ToString();
            content.MetaDescription = row["MetaDescription"].ToString();
            content.SubscriptionLevelID = row["SubscriptionLevelID"].ToString();
            content.UrlName = row["UrlName"].ToString();

            return content;
        }

        private IList<Content> ConvertTableToList( Culture culture, DataTable table )
        {
            return ConvertTableToList( culture, table, 0, table.Rows.Count );
        }

        private IList<Content> ConvertTableToList( Culture culture, DataTable table, int startIndex, int endIndex )
        {
            IList<Content> list = new List<Content>();

            for (int i = startIndex; i <= endIndex && i < table.Rows.Count; i++)
                list.Add( CreateOneFromRow( culture, table.Rows[i] ) );

            return list;
        }

        private string GetCultureID( Culture culture )
        {
            return culture.CultureID;
        }

        private static string ConditionContentMenuItemID( string contentMenuItemIDList, ArrayList parameters )
        {
            if (!String.IsNullOrEmpty( contentMenuItemIDList ))
            {
                string query = " AND C.ContentMenuItemID IN (";

                string[] contentMenuItemIDs = contentMenuItemIDList.Trim().Split( ' ' );
                for (int i = 0; i < contentMenuItemIDs.Length; i++)
                {
                    if (!string.IsNullOrEmpty( contentMenuItemIDs[i].Trim() ))
                    {
                        query += "@ContentMenuItemID" + i + ",";
                        parameters.Add( DataAccess.CreateParameterString( contentMenuItemIDs[i] ) );
                    }
                }
                query = query.TrimEnd( ',' ) + ") ";

                return query;
            }
            else
                return "";
        }

        private string ConditionSearchField(
            string searchTerm,
            string[] searchFieldList,
            ArrayList parameters,
            string searchType )
        {
            string query = "";

            if (!String.IsNullOrEmpty( searchTerm ))
            {
                switch (searchType)
                {
                    case "all":
                        query = GetAllWordsSearch( searchTerm, searchFieldList, parameters );
                        break;
                    case "exact":
                        query = GetExactPhaseSearch( searchTerm, searchFieldList, parameters );
                        break;
                    default:
                        query = GetAnyWordSearch( searchTerm, searchFieldList, parameters );
                        break;
                }

                return query;
            }
            else
                return "";
        }

        private string GetAllWordsSearch( string searchTerm, string[] searchFieldList, ArrayList parameters )
        {
            string[] arKeys = StringUtilities.SplitString( searchTerm, new char[] { ' ', '@', ';', '\'' } );
            string query = " AND (";

            for (int i = 0; i < searchFieldList.Length; i++)
            {
                for (int j = 0; j < arKeys.Length; j++)
                {
                    if (j == 0)
                    {
                        query += " ( ";
                    }
                    else
                    {
                        query += " AND ";
                    }

                    string strColumn = searchFieldList[i];
                    query += CreateColumnName( strColumn );

                    parameters.Add( DataAccess.CreateParameterString( "%" + arKeys[j] + "%" ) );
                    query += " LIKE @" + strColumn + j;

                    if (j == arKeys.Length - 1)
                    {
                        query += " ) ";
                    }
                }

                if (i != searchFieldList.Length - 1)
                {
                    query += " OR ";
                }
            }

            query += " ) ";

            return query;
        }

        private string GetExactPhaseSearch( string searchTerm, string[] searchFieldList, ArrayList parameters )
        {
            string query = " AND (";

            for (int i = 0; i < searchFieldList.Length; i++)
            {
                string strColumn = searchFieldList[i];

                if (i == 0)
                    query += " ";
                else
                    query += " OR ";

                query += CreateColumnName( strColumn );

                parameters.Add( DataAccess.CreateParameterString( "%" + searchTerm + "%" ) );
                query += " LIKE @" + strColumn;
            }

            query += " ) ";

            return query;
        }

        private string GetAnyWordSearch( string searchTerm, string[] searchFieldList, ArrayList parameters )
        {
            string[] arKeys = StringUtilities.SplitString( searchTerm, new char[] { ' ', '@', ';', '\'' } );
            string query = "";

            for (int i = 0; i < arKeys.Length; i++)
            {
                if (i == 0)
                {
                    query = " AND (";
                }

                for (int j = 0; j < searchFieldList.Length; j++)
                {
                    string strColumn = searchFieldList[j];
                    if (j == 0 && i == 0)
                        query += " ";
                    else
                        query += " OR ";

                    query += CreateColumnName( strColumn );

                    parameters.Add( DataAccess.CreateParameterString( "%" + arKeys[i] + "%" ) );
                    query += " LIKE @" + strColumn + i;
                }

                if (i == arKeys.Length - 1)
                    query += " ) ";
            }

            return query;
        }

        private string CreateColumnName( string columnName )
        {
            if (String.Compare( columnName, "Other1", true ) == 0 ||
                String.Compare( columnName, "Other2", true ) == 0 ||
                String.Compare( columnName, "Other3", true ) == 0 ||
                String.Compare( columnName, "IsEnabled", true ) == 0)
            {
                return "Content." + columnName;
            }
            else
            {
                return columnName;
            }
        }

        private void CopyContentLocales( Content content, IList<ILocale> locales )
        {
            foreach (ContentLocale locale in locales)
            {
                content.Locales.Add( locale );
            }
        }

        #endregion


        #region Public Methods

        public Content Save( Content content )
        {
            Content result;

            using (DataTransactionScope transaction = new DataTransactionScope())
            {
                if (content.ContentID == "0")
                    result = Create( content );
                else
                    result = Update( content );

                transaction.Complete();
            }

            return result;
        }

        public Content GetOne( Culture culture, string contentID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllFields +
                "FROM Content " +
                "LEFT JOIN (SELECT * FROM ContentLocale WHERE CultureID = @CultureID) L " +
                "ON Content.ContentID = L.ContentID " +
                "WHERE Content.ContentID = @ContentID; ",
                DataAccess.CreateParameterString( RepositoryUtilities.GetCultureID( culture ) ),
                DataAccess.CreateParameterString( contentID )
                );

            if (table.Rows.Count > 0)
            {
                Content content = CreateOneFromRow( culture, table.Rows[0] );
                CopyContentLocales( content, GetLocales( contentID ) );
                return content;
            }
            else
                return Content.Null;
        }

        public string GetContentIDFromUrlName( string urlName )
        {
            string contentID = DataAccess.ExecuteScalar(
                "SELECT ContentID " +
                "FROM ContentLocale " +
                "WHERE UrlName = @UrlName; ",
                DataAccess.CreateParameterString( urlName ) );

            return contentID;
        }

        public string GetIDByName( Culture culture, string contentName )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllFields +
                "FROM Content " +
                "LEFT JOIN (SELECT * FROM ContentLocale WHERE CultureID = @CultureID) L " +
                "ON Content.ContentID = L.ContentID " +
                "WHERE Content.ContentName = @ContentName; ",
                DataAccess.CreateParameterString( RepositoryUtilities.GetCultureID( culture ) ),
                DataAccess.CreateParameterString( contentName )
                );

            if (table.Rows.Count > 0)
                return table.Rows[0]["ContentID"].ToString();
            else
                return "0";
        }

        public void Delete( string contentID )
        {
            using (DataTransactionScope transaction = new DataTransactionScope())
            {
                DataAccess.ExecuteNonQuery(
                "DELETE " +
                "FROM Content " +
                "WHERE ContentID = @ContentID; ",
                DataAccess.CreateParameterString( contentID ) );

                DeleteLocales( contentID );
                transaction.Complete();
            }
        }

        public void DeleteLocalesByCultureID( string cultureID )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE " +
                "FROM ContentLocale " +
                "WHERE CultureID = @CultureID ;",
                DataAccess.CreateParameterString( cultureID ) );
        }

        public IList<ILocale> GetLocales( string contentID )
        {
            DataTable localeTable = DataAccess.ExecuteSelect(
               "SELECT " + AllContentLocaleFields +
               "FROM ContentLocale " +
               "WHERE ContentID = @ContentID; ",
               DataAccess.CreateParameterString( contentID ) );

            return ConvertLocaleTableToList( localeTable );
        }

        public IList<Content> GetAll( Culture culture, BoolFilter showInSiteMap, string sortBy )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllFields +
                "FROM Content LEFT JOIN " +
                "(SELECT * FROM ContentLocale WHERE CultureID = @CultureID) AS L " +
                "ON Content.ContentID = L.ContentID " +
                FlagFilterHelper.CreateFlagFilterWithWhere(
                    "IsShowInSiteMap",
                    RepositoryUtilities.ConvertToFlagFilter( showInSiteMap ) ) +
                    CreateSortByText( sortBy ) + "; ",
                DataAccess.CreateParameterString( RepositoryUtilities.GetCultureID( culture ) )
                );

            return ConvertTableToList( culture, table );
        }

        public IList<Content> SearchContent(
            Culture culture,
            string sortBy,
            SearchFilter searchFilter,
            int startIndex,
            int endIndex,
            out int howManyItems
            )
        {
            ArrayList parameterList = new ArrayList();
            parameterList.Add( DataAccess.CreateParameterString( RepositoryUtilities.GetCultureID( culture ) ) );
            SearchFilterHelperNew searchFilterHelper = new SearchFilterHelperNew();
            string commandSearch = searchFilterHelper.CreateCommandWithWhere(
                searchFilter, ConvertFieldName( searchFilter.FieldName ), parameterList );

            DataTable full = DataAccess.ExecuteSelect(
                "SELECT " + AllFields +
                "FROM Content LEFT JOIN " +
                "     (SELECT * FROM ContentLocale WHERE CultureID = @CultureID) AS L " +
                "     ON Content.ContentID = L.ContentID " +
                commandSearch +
                CreateSortByText( sortBy ) + "; ",
                DataAccess.ConvertArrayListToDbParameters( parameterList )
                );

            howManyItems = full.Rows.Count;

            DataColumn[] keys = new DataColumn[] { full.Columns["ContentID"] };
            full.PrimaryKey = keys;
            return ConvertTableToList( culture, full, startIndex, endIndex );
        }

        public IList<TableSchemaItem> GetTableSchema()
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllFields +
                "FROM Content INNER JOIN ContentLocale L " +
                "ON Content.ContentID = L.ContentID WHERE  1 = 0; "
                );

            return RepositoryUtilities.ConvertTableToSchemaItemList( table );
        }

        public IList<Content> AdvancedSearch(
            Culture culture,
            string storeID,
            string contentMenuItemIDs,
            string sortBy,
            string searchTerm,
            string[] searchFieldList,
            int startIndex,
            int endIndex,
            string searchType,
            out int howManyItems )
        {
            ArrayList parameterList = new ArrayList();
            parameterList.Add( DataAccess.CreateParameterString( storeID ) );
            parameterList.Add( DataAccess.CreateParameterString( RepositoryUtilities.GetCultureID( culture ) ) );

            DataTable full = DataAccess.ExecuteSelect(
                "SELECT DISTINCT" + AllFields + "FROM ((Content LEFT JOIN ContentLocale L ON Content.ContentID = L.ContentID)" +
                "RIGHT JOIN (SELECT ContentMenuItemID, ContentID AS CCID, StoreID FROM ContentMenuItem WHERE StoreID = @StoreID)" +
                "AS C ON C.CCID = Content.ContentID )" +
                "WHERE CultureID = @CultureID " +
                 FlagFilterHelper.CreateFlagFilterWithAnd( "IsEnabled", RepositoryUtilities.ConvertToFlagFilter( BoolFilter.ShowTrue ) ) +
                 ConditionSearchField( searchTerm, searchFieldList, parameterList, searchType ) +
                 ConditionContentMenuItemID( contentMenuItemIDs, parameterList ) +
                 CreateSortByText( sortBy ) + ";",
                 DataAccess.ConvertArrayListToDbParameters( parameterList )
                );

            howManyItems = full.Rows.Count;
            return ConvertTableToList( culture, full, startIndex, endIndex );
        }
        #endregion
    }
}
