using System;
using System.Collections.Generic;
using System.Text;
using System.Web.UI;

using Vevo.Base.Data.Caching;
using Vevo.Domain;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.Discounts;
using Vevo.Shared.Utilities;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class DiscountGroupRepository : IDiscountGroupRepository
    {
        #region Private

        private const string RepositoryName = "DiscountGroupRepository";

        private const string GetOneSignature = "GetOne";
        private const string GetDiscountRulesByDiscountGroupIDSignature = "GetDiscountRulesByDiscountGroupID";
        private const string GetAllSignature = "GetAll";
        private const string GetTableSchemaSignature = "GetTableSchema";
        private const string SearchDiscountGroupSignature = "SearchDiscountGroup";
        private DataCache _dataCache = new DataCache( RepositoryName );

        private ICacheDependencyUtility _cacheDependency = new CacheDependencyUtility( new string[] { "DiscountGroup", "DiscountRule" } );
        private DiscountGroupRepositoryNoCache _repository = new DiscountGroupRepositoryNoCache();

        #endregion

        #region Constructors

        public DiscountGroupRepository()
        {
        }

        public DiscountGroupRepository( ICacheDependencyUtility cacheDependency )
        {
            _cacheDependency = cacheDependency;
        }

        #endregion

        // Functions that are intended to be used internally inside aggregates.
        #region Aggregate Methods

        public IList<DiscountRule> GetDiscountRulesByDiscountGroupID( string discountGroupID )
        {
            IList<DiscountRule> list =
                (IList<DiscountRule>) _dataCache.GetValue(
                    GetDiscountRulesByDiscountGroupIDSignature,
                    _cacheDependency,
                    discountGroupID );

            if (list == null)
            {
                list = _repository.GetDiscountRulesByDiscountGroupID( discountGroupID );
                _dataCache.SetValue( GetDiscountRulesByDiscountGroupIDSignature, list, discountGroupID );
            }

            return list;
        }

        #endregion


        #region Public Methods

        public DiscountGroup Save( DiscountGroup discountGroup )
        {
            // Need to write to database first before clearing cache.
            // Otherwise, cache may remember incorrect values before saving.
            DiscountGroup newDiscountGroup = _repository.Save( discountGroup );
            _dataCache.ClearRepositoryCache();
            return newDiscountGroup;
        }

        public DiscountGroup GetOne( string discountGroupID )
        {
            DiscountGroup result = (DiscountGroup) _dataCache.GetValue( GetOneSignature, _cacheDependency, discountGroupID );

            if (result == null)
            {
                result = _repository.GetOne( discountGroupID );
                _dataCache.SetValue( GetOneSignature, result, discountGroupID );
            }

            return result;
        }

        public void Delete( string discountGroupID )
        {
            _repository.Delete( discountGroupID );
            _dataCache.ClearRepositoryCache();
        }

        public IList<DiscountGroup> GetAll()
        {
            IList<DiscountGroup> result = (IList<DiscountGroup>) _dataCache.GetValue( GetAllSignature, _cacheDependency );

            if (result == null)
            {
                result = _repository.GetAll();
                _dataCache.SetValue( GetAllSignature, result );
            }

            return result;
        }

        public IList<TableSchemaItem> GetTableSchema()
        {
            IList<TableSchemaItem> list = (IList<TableSchemaItem>) _dataCache.GetValue( GetTableSchemaSignature, _cacheDependency );

            if (list == null)
            {
                list = _repository.GetTableSchema();
                _dataCache.SetValue( GetTableSchemaSignature, list );
            }

            return list;
        }

        public IList<DiscountGroup> SearchDiscountGroup(
            string sortBy,
            SearchFilter searchFilter,
            int startIndex,
            int endIndex,
            out int howManyItems )
        {
            howManyItems = 0;
            Pair result = (Pair) _dataCache.GetValue(
                SearchDiscountGroupSignature,
                _cacheDependency,
                sortBy,
                searchFilter.ToString(),
                startIndex,
                endIndex
                );

            if (result == null)
            {
                result = new Pair();
                result.First = _repository.SearchDiscountGroup(
                    sortBy, searchFilter, startIndex, endIndex, out howManyItems );
                result.Second = howManyItems;
                _dataCache.SetValue(
                    SearchDiscountGroupSignature,
                    result,
                    sortBy,
                    searchFilter.ToString(),
                    startIndex,
                    endIndex );
            }

            howManyItems = ConvertUtilities.ToInt32( result.Second );
            return (IList<DiscountGroup>) result.First;
        }

        #endregion
    }
}
