﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using Vevo.Data.Implementation;
using Vevo.Domain;
using Vevo.Domain.Base;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.Products;
using Vevo.Shared.DataAccess;
using Vevo.Shared.Utilities;
using Vevo.Base.Data;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class SpecificationItemRepositoryNoCache : ISpecificationItemRepository
    {
        #region Private

        private const string AllSpecificationItemLocaleFields = " SpecificationItemID , CultureID, DisplayName, Description  ";
        private const string AllSpecificationFields =
           " SpecificationItem.SpecificationItemID, SpecificationItem.SpecificationGroupID, Name, DisplayName, " +
           "Description, Type, UseInFacetedSearch ";
        private ProductSpecificationAccess _productSpecificationAccess = new ProductSpecificationAccess();
        private GoogleSpecMappingAccess _googleSpecMappingAccess = new GoogleSpecMappingAccess();

        private SpecificationItem CreateOneFromRow( Culture culture, DataRow row )
        {
            SpecificationItem specificationItem = new SpecificationItem( culture );

            specificationItem.SpecificationItemID = row["SpecificationItemID"].ToString();
            specificationItem.Name = row["Name"].ToString();
            specificationItem.SpecificationGroupID = row["SpecificationGroupID"].ToString();
            specificationItem.Type = (SpecificationItemControlType) Enum.Parse( typeof( SpecificationItemControlType ), row["Type"].ToString() );
            specificationItem.UseInFacetedSearch = ConvertUtilities.ToBoolean( row["UseInFacetedSearch"] );

            return specificationItem;
        }

        private SpecificationItemLocale CreateOneLocaleFromRow( DataRow row )
        {
            SpecificationItemLocale locale = new SpecificationItemLocale();
            locale.CultureID = row["CultureID"].ToString();
            locale.DisplayName = row["DisplayName"].ToString();
            locale.Description = row["Description"].ToString();

            return locale;
        }

        private void CopySpecificationItemLocales( SpecificationItem specificationItem, IList<ILocale> locales )
        {
            foreach (SpecificationItemLocale locale in locales)
            {
                specificationItem.Locales.Add( locale );
            }
        }

        private IList<SpecificationItem> ConvertSpecificationItemTableToList( Culture culture, DataTable table )
        {
            IList<SpecificationItem> list = new List<SpecificationItem>();

            foreach (DataRow row in table.Rows)
                list.Add( CreateOneFromRow( culture, row ) );

            return list;
        }

        private string ConvertFieldName( string fieldName )
        {
            switch (fieldName.ToLower())
            {
                case "specificationitemid":
                    return "SpecificationItem.SpecificationItemID";
                default:
                    return fieldName;
            }
        }

        private string CreateSortByText( string sortBy )
        {
            if (sortBy == null || sortBy.Trim() == "")
                return "";
            else if (sortBy.Trim().ToLower().StartsWith( "specificationitemid" ))
                return "ORDER BY SpecificationItem." + sortBy.Trim() + " ";
            else
                return "ORDER BY " + sortBy + " ";
        }

        private List<ILocale> ConvertLocaleTableToList( DataTable localeTable )
        {
            List<ILocale> list = new List<ILocale>();

            foreach (DataRow row in localeTable.Rows)
                list.Add( CreateOneLocaleFromRow( row ) );

            return list;
        }

        private IList<SpecificationItem> ConvertTableToList( Culture culture, DataTable table, int startIndex, int endIndex )
        {
            IList<SpecificationItem> list = new List<SpecificationItem>();

            for (int i = startIndex; i <= endIndex && i < table.Rows.Count; i++)
                list.Add( CreateOneFromRow( culture, table.Rows[i] ) );

            return list;
        }

        private void DeleteLocales( string specificationItemID )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE FROM SpecificationItemLocale " +
                "WHERE SpecificationItemID = @SpecificationItemID; ",
                DataAccess.CreateParameterString( specificationItemID ) );
        }

        private void InsertLocales( SpecificationItem specificationItem )
        {
            foreach (SpecificationItemLocale locale in specificationItem.Locales)
            {
                DataAccess.ExecuteNonQuery(
                    "INSERT INTO SpecificationItemLocale ( SpecificationItemID, CultureID, DisplayName , Description) " +
                    "VALUES ( @SpecificationItemID, @CultureID, @DisplayName, @Description )",
                    DataAccess.CreateParameterString( specificationItem.SpecificationItemID ),
                    DataAccess.CreateParameterString( locale.CultureID ),
                    DataAccess.CreateParameterString( locale.DisplayName ),
                    DataAccess.CreateParameterString( locale.Description ) );
            }
        }

        private SpecificationItem Create( SpecificationItem specificationItem )
        {
            string newID = DataAccess.ExecuteScalar(
                "INSERT INTO SpecificationItem ( SpecificationGroupID, Name, Type, UseInFacetedSearch ) " +
                "VALUES ( @SpecificationGroupID, @Name, @Type, @UseInFacetedSearch);" +
                "SELECT @@Identity;",
                DataAccess.CreateParameterString( specificationItem.SpecificationGroupID ),
                DataAccess.CreateParameterString( specificationItem.Name ),
                DataAccess.CreateParameterString( specificationItem.Type.ToString() ),
                DataAccess.CreateParameterBool( specificationItem.UseInFacetedSearch ) );

            specificationItem.SpecificationItemID = newID;

            InsertLocales( specificationItem );

            return specificationItem;
        }

        private SpecificationItem Update( SpecificationItem specificationItem )
        {
            DataAccess.ExecuteNonQuery(
                "UPDATE SpecificationItem " +
                "SET SpecificationGroupID = @SpecificationGroupID, Name = @Name,  " +
                "Type = @Type, UseInFacetedSearch = @UseInFacetedSearch " +
                "WHERE SpecificationItemID = @SpecificationItemID ;",
                DataAccess.CreateParameterString( specificationItem.SpecificationGroupID ),
                DataAccess.CreateParameterString( specificationItem.Name ),
                DataAccess.CreateParameterString( specificationItem.Type.ToString() ),
                DataAccess.CreateParameterBool( specificationItem.UseInFacetedSearch ),
                DataAccess.CreateParameterString( specificationItem.SpecificationItemID ) );

            DeleteLocales( specificationItem.SpecificationItemID );
            InsertLocales( specificationItem );

            return specificationItem;
        }
        #endregion

        #region Public

        public SpecificationItem Save( SpecificationItem specificationItem )
        {
            SpecificationItem result;

            using (DataTransactionScope transaction = new DataTransactionScope())
            {
                if (specificationItem.SpecificationItemID == "0")
                    result = Create( specificationItem );
                else
                    result = Update( specificationItem );

                transaction.Complete();
            }

            return result;
        }
        public IList<ILocale> GetLocales( string specificationItemID )
        {
            DataTable localeTable = DataAccess.ExecuteSelect(
                "SELECT " + AllSpecificationItemLocaleFields +
                "FROM SpecificationItemLocale " +
                "WHERE SpecificationItemID  = @SpecificationItemID ; ",
                DataAccess.CreateParameterString( specificationItemID ) );

            return ConvertLocaleTableToList( localeTable );
        }

        public IList<TableSchemaItem> GetTableSchema()
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT SpecificationItem.SpecificationItemID,  Name, DisplayName, " +
                "Description, Type, UseInFacetedSearch " +
                "FROM SpecificationItem LEFT JOIN " +
                "(SELECT * FROM SpecificationItemLocale) AS L " +
                "ON SpecificationItem.SpecificationItemID = L.SpecificationItemID " +
                "WHERE 1 = 0 ;" );

            return RepositoryUtilities.ConvertTableToSchemaItemList( table );
        }


        public void Delete( string specificationItemID )
        {
            using (DataTransactionScope transaction = new DataTransactionScope())
            {
                DataAccessContext.SpecificationItemValueRepository.DeleteBySpecificationItemID( specificationItemID );

                DataAccess.ExecuteNonQuery(
                 "DELETE FROM SpecificationItem WHERE SpecificationItemID = @SpecificationItemID;" +
                 "DELETE FROM ProductSpecification WHERE SpecificationItemID = @SpecificationItemID;",
                DataAccess.CreateParameterString( specificationItemID ),
                DataAccess.CreateParameterString( specificationItemID ) );
                DeleteLocales( specificationItemID );
                _googleSpecMappingAccess.Delete( specificationItemID );
                transaction.Complete();
            }
        }

        public void DeleteBySpecificationGroupID( string specificationGroupID )
        {
            DataTable table = DataAccess.ExecuteSelect(
               "SELECT * FROM SpecificationItem WHERE SpecificationGroupID = @SpecificationGroupID;",
               DataAccess.CreateParameterString( specificationGroupID ) );

            foreach (DataRow row in table.Rows)
            {
                Delete( row["SpecificationItemID"].ToString() );
            }
        }

        public void DeleteLocalesByCultureID( string cultureID )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE " +
                "FROM SpecificationItemLocale " +
                "WHERE CultureID = @CultureID ;",
                DataAccess.CreateParameterString( cultureID ) );
        }

        public DataTable GetAll( Culture culture )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT SpecificationItem.SpecificationItemID, SpecificationItem.SpecificationGroupID, " +
                "SpecificationItem.Name, ItemLocale.DisplayName, ItemLocale.Description, SpecGroup.Name AS GroupName, " +
                "SpecificationItem.Type, SpecificationItem.UseInFacetedSearch " +
                "FROM SpecificationItem " +
                "LEFT JOIN (SELECT * FROM SpecificationItemLocale WHERE CultureID = @CultureID) AS ItemLocale " +
                "ON SpecificationItem.SpecificationItemID = ItemLocale.SpecificationItemID " +
                "LEFT JOIN " +
                "(select SpecificationGroup.Name, SpecificationGroup.SpecificationGroupID from SpecificationGroup " +
                " ) AS SpecGroup " +
                "ON SpecificationItem.SpecificationGroupID = SpecGroup.SpecificationGroupID " +
                "ORDER BY SpecificationItem.SpecificationGroupID ",
                DataAccess.CreateParameterString( culture.CultureID ) );

            return table;
        }

        public IList<ProductSpecification> GetAllProductSpecificationValueByItemID( string specificationItemID )
        {
            return _productSpecificationAccess.GetAllProductSpecificationValueByItemID( specificationItemID );
        }

        public IList<ProductSpecification> GetAllProductSpecificationValueByItemIDAndValue( string specificationItemID, string specificationItemValue )
        {
            return _productSpecificationAccess.GetAllProductSpecificationValueByItemIDAndValue( specificationItemID, specificationItemValue );
        }

        public SpecificationItem GetOne( Culture culture, string specificationItemID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllSpecificationFields +
                "FROM SpecificationItem LEFT JOIN " +
                "    (SELECT * FROM SpecificationItemLocale WHERE CultureID = @CultureID) AS L " +
                "ON SpecificationItem.SpecificationItemID = L.SpecificationItemID " +
                "WHERE SpecificationItem.SpecificationItemID = @SpecificationItemID; ",
                DataAccess.CreateParameterString( culture.CultureID ),
                DataAccess.CreateParameterString( specificationItemID ) );

            if (table.Rows.Count > 0)
            {
                SpecificationItem specificationItem = CreateOneFromRow( culture, table.Rows[0] );
                CopySpecificationItemLocales( specificationItem, GetLocales( specificationItemID ) );
                return specificationItem;
            }
            else
                return SpecificationItem.Null;
        }

        public SpecificationItem GetOneByName( Culture culture, string name )
        {
            DataTable table = DataAccess.ExecuteSelect(
               "SELECT " + AllSpecificationFields +
               "FROM SpecificationItem LEFT JOIN " +
               "    (SELECT * FROM SpecificationItemLocale WHERE CultureID = @CultureID) AS L " +
               "ON SpecificationItem.SpecificationItemID = L.SpecificationItemID " +
               "WHERE Name = @Name; ",
               DataAccess.CreateParameterString( culture.CultureID ),
               DataAccess.CreateParameterString( name ) );

            if (table.Rows.Count > 0)
            {
                SpecificationItem specificationItem = CreateOneFromRow( culture, table.Rows[0] );
                CopySpecificationItemLocales( specificationItem, GetLocales( specificationItem.SpecificationItemID ) );
                return specificationItem;
            }
            else
                return SpecificationItem.Null;
        }

        public IList<SpecificationItem> GetBySpecificationGroupID( Culture culture, string specificationGroupID )
        {
            DataTable table = DataAccess.ExecuteSelect(
               "SELECT " + AllSpecificationFields +
               "FROM SpecificationItem LEFT JOIN " +
               "    (SELECT * FROM SpecificationItemLocale WHERE CultureID = @CultureID) AS L " +
               "ON SpecificationItem.SpecificationItemID = L.SpecificationItemID " +
               "WHERE SpecificationGroupID = @SpecificationGroupID; ",
               DataAccess.CreateParameterString( culture.CultureID ),
               DataAccess.CreateParameterString( specificationGroupID ) );

            return ConvertSpecificationItemTableToList( culture, table );
        }

        public IList<SpecificationItem> SearchSpecificationItem(
            Culture culture,
            string specificationGroupID,
            string sortBy,
            SearchFilter searchfilter,
            int startIndex,
            int endIndex,
            out int howManyItems )
        {
            string sqlFieldName = ConvertFieldName( searchfilter.FieldName );
            ArrayList parameterList = new ArrayList();
            parameterList.Add( DataAccess.CreateParameterString( RepositoryUtilities.GetCultureID( culture ) ) );
            SearchFilterHelperNew searchFilterHelper = new SearchFilterHelperNew();
            string commandSearch = searchFilterHelper.CreateCommandWithWhere( searchfilter, sqlFieldName, parameterList );
            parameterList.Add( DataAccess.CreateParameterString( specificationGroupID ) );

            string query = "SELECT ROW_NUMBER() over ( " + CreateSortByText( sortBy ) + ") as rownum, " +
                " * FROM (" +
                "SELECT SpecificationItem.SpecificationItemID, SpecificationGroupID, Name, Type, UseInFacetedSearch " +
                "FROM SpecificationItem LEFT JOIN " +
                "(SELECT * FROM SpecificationItemLocale WHERE CultureID = @CultureID) AS L " +
                "ON SpecificationItem.SpecificationItemID = L.SpecificationItemID " +
                commandSearch +
                "AND SpecificationGroupID = @SpecificationGroupID " +
                ") AS SpecificationItem ";

            string extQuery = "with SearchResult as (" + query + ") " +
                    " select * from SearchResult where rownum between " + (startIndex + 1).ToString() + " and " + (endIndex + 1).ToString() + ";";

            DataTable table = DataAccess.ExecuteSelect( extQuery,
            DataAccess.ConvertArrayListToDbParameters( parameterList ) );

            howManyItems = table.Rows.Count;

            //DataColumn[] keys = new DataColumn[] { table.Columns["OptionItem.OptionItemID"] };
            //table.PrimaryKey = keys;
            return ConvertTableToList( culture, table, startIndex, endIndex );
        }

         public bool IsSpecificationItemNameExist(string name, string specificationGroupID)
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT SpecificationItemID FROM SpecificationItem "+
                "WHERE SpecificationGroupID = @ID "+
                "AND Name = @Name; ",
                DataAccess.CreateParameterString(specificationGroupID),
                DataAccess.CreateParameterString( name ) );

            if (table.Rows.Count > 0)
            {
                return true;
            }
            return false;
        }

        #endregion

    }
}
